from pyramid.config import Configurator
from charmworld.resources import Root
from charmworld.globals import default_globals_factory


def main(global_config, **settings):
    """ This function returns a Pyramid WSGI application.
    """
    config = Configurator(root_factory=Root, settings=settings)

#    config.add_view('charmworld.views.home',
#                    context='charmworld:resources.Root',
#                    renderer='charmworld:templates/index.pt')

    # Template globals
    config.set_renderer_globals_factory(default_globals_factory)

    config.add_static_view('static', 'charmworld:static', cache_max_age=3600)

    config.add_route("charm-collection", "/charms")
    config.add_route("series-collection", "/charms/{series}")
    config.add_route("charm", "/charms/{series}/{charm}")
    config.add_route("hook", "/charms/{series}/{charm}/hooks/{hook}")
    config.add_route("config", "/charms/{series}/{charm}/config")

    config.add_route("personal-collection", "/~{owner}")
    config.add_route("personal-series", "/~{owner}/{series}")
    config.add_route("personal-charm", "/~{owner}/{series}/{charm}")
    config.add_route(
        "personal-hook", "/~{owner}/{series}/{charm}/hooks/{hook}")
    config.add_route(
        "personal-config", "/~{owner}/{series}/{charm}/config")

    config.add_route("interface", "/interfaces/{interface}")
    config.add_route("interface-collection", "/interfaces")

    config.add_route("search", "/search")
    config.add_route("home", "/")
    config.scan("charmworld")

    return config.make_wsgi_app()
