import json
import logging
import os
import redis
import subprocess


log = logging.getLogger("charm.bzr")


def fetch_branch(root_dir, charm_data):
    """Fetch a branch from bzr.
    """
    branch_dir = os.path.abspath(
        str(os.path.join(root_dir,
                         charm_data["owner"],
                         charm_data["name"],
                         charm_data["bname"])))

    if not os.path.exists(os.path.dirname(branch_dir)):
        os.makedirs(os.path.dirname(branch_dir))

    # Store the branch directory
    charm_data["branch_dir"] = branch_dir

    if not os.path.exists(branch_dir):
        log.info("Branching charm lp:%s", charm_data["branch_spec"])
        subprocess.check_output(
            ["/usr/bin/bzr", "co",
             "lp:%s" % charm_data["branch_spec"], branch_dir])
        return
    else:
        log.debug("Existing charm from lp:%s", charm_data["branch_spec"])
        output = subprocess.check_output(
            ["/usr/bin/bzr", "revision-info"],
            cwd=branch_dir,
            stderr=subprocess.STDOUT)
        revno, revid = output.strip().split()

        if revid == charm_data["commit"]:
            log.debug("Already up to date lp:%s", charm_data["branch_spec"])
            return
    print "rev stat %r %r %r" % (revno, revid, charm_data["commit"])
    log.debug("Updating branch lp:%s", charm_data["branch_spec"])
    subprocess.check_output(["/usr/bin/bzr", "update"], cwd=branch_dir)


def fetch_branches(
    server, root_dir,
    queue="bzr-fetch", scan_queue="charm-log", error_queue="fetch-err"):
    """
    """
    while 1:
        item = server.spop(queue)
        if not item:
            return
        try:
            charm_data = json.loads(item)
            fetch_branch(root_dir, charm_data)
        except Exception, e:
            log.exception("bzr error error on %s: %s", item, str(e))
            charm_data["error_stage"] = "bzr"
            charm_data["error"] = str(e.output)
            server.lpush(
                error_queue, json.dumps(charm_data))
            continue
        else:
            server.lpush(scan_queue, json.dumps(charm_data))


def main():
    logging.basicConfig(level=logging.DEBUG)
    root_dir = os.path.abspath(
        os.path.expanduser("~/projects/charmworld/repo"))
    server = redis.Redis()
    fetch_branches(server, root_dir)


if __name__ == '__main__':
    main()
