from datetime import datetime
import json
import logging
import os
import redis

from bzrlib.transport import get_transport
from bzrlib.branch import Branch
from bzrlib.revisionspec import RevisionSpec

log = logging.getLogger("charm.changelog")


def _rev_info(r, branch):
    d = {
        "revno": branch.revision_id_to_revno(r.revision_id),
        "committer": r.committer,
        "created": r.timestamp,
        "message": r.message
        }
    return d


def fetch_changes(charm_data):
    branch_dir = charm_data["branch_dir"]
    transport = get_transport(branch_dir)
    branch = Branch.open_from_transport(transport)

    # We only want the last 10 changes, in descending order.
    _, cur_rev_id = branch.last_revision_info()
    spec = RevisionSpec.from_string("revno:-10")
    last_rev_id = spec.as_revision_id(branch)

    branch.lock_read()
    try:
        graph = branch.repository.get_graph()
        revs = list(
            graph.iter_lefthand_ancestry(cur_rev_id, (last_rev_id,)))
        revs.insert(0, cur_rev_id)
    finally:
        branch.unlock()
    revisions = map(branch.repository.get_revision, revs)

    charm_data["changes"] = changes = []

    for r in revisions:
        changes.append(_rev_info(r, branch))
    charm_data["last_change"] = changes[0]


def scan_changes(server, in_queue="charm-log", out_queue="charm-scan"):

    while 1:
        item = server.lpop(in_queue)
        if not item:
            return

        try:
            charm_data = json.loads(item)
            log.info("Fetching changes for %s", charm_data["branch_spec"])
            fetch_changes(charm_data)
        except Exception, e:
            import pdb, traceback, sys
            traceback.print_exc()
            pdb.post_mortem(sys.exc_info()[-1])
            log.exception("bzr error on %s: %s", item, str(e))
            raise
        else:
            server.lpush(out_queue, json.dumps(charm_data))


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    server = redis.Redis()
    scan_changes(server)


    import time
    import pprint
    s = time.time()
    charm_data = {}
    charm_data["branch_dir"] = os.path.expanduser(
        "~/projects/charmworld/repo/charmers/"
        "nova-cloud-controller/trunk")
    #changes = list(fetch_changes(charm_data))
    #print "Finished", time.time() - s
    #pprint.pprint(changes)
